# !/usr/bin/env python3
# -*- coding:utf-8 -*-
import asyncio
import os
from contextlib import AsyncExitStack, ExitStack
# @Time    : 2024/3/11 16:02
# @Author  : fanen.lhy
# @Email   : fanen.lhy@antgroup.com
# @FileName: mcp_session_manager.py
from contextvars import ContextVar
from types import TracebackType
from typing import Callable, Any
from typing import Literal, cast
from datetime import timedelta

from anyio.from_thread import start_blocking_portal
from mcp import StdioServerParameters, stdio_client, ClientSession
from mcp.client.sse import sse_client
from mcp.client.streamable_http import streamablehttp_client

from agentuniverse.base.annotation.singleton import singleton

EncodingErrorHandler = Literal["strict", "ignore", "replace"]

DEFAULT_ENCODING = "utf-8"
DEFAULT_ENCODING_ERROR_HANDLER: EncodingErrorHandler = "strict"

DEFAULT_HTTP_TIMEOUT = 5
DEFAULT_SSE_READ_TIMEOUT = 60 * 5

DEFAULT_STREAMABLE_HTTP_TIMEOUT = timedelta(seconds=30)
DEFAULT_STREAMABLE_HTTP_SSE_READ_TIMEOUT = timedelta(seconds=60 * 5)

class SyncAsyncExitStack:
    def __init__(self) -> None:
        self._portal_cm = start_blocking_portal()
        self._portal = self._portal_cm.__enter__()
        self._stack: ExitStack = ExitStack()

    def run_async(self, func, *args, **kwargs):
        """use portal to run async code"""
        return self._portal.call(func, *args, **kwargs)

    def enter_async_context(self, async_cm):
        sync_cm = self._portal.wrap_async_context_manager(async_cm)
        return self._stack.enter_context(sync_cm)

    def callback(self, func: Callable, *args, **kwargs):
        return self._portal.call(self._stack.callback, func, *args, **kwargs)

    def close(self):
        try:
            self._stack.close()
        finally:
            self._portal_cm.__exit__(None, None, None)


def pick_exit_stack():
    """
    return SyncAsyncExitStack in sync mode otherwise AsyncExitStack
    """
    try:
        asyncio.get_running_loop()
        return AsyncExitStack()
    except RuntimeError:
        return SyncAsyncExitStack()


class MCPTempClient:
    def __init__(self, connection_args: dict):
        self.exit_stack = AsyncExitStack()
        self.connection_args = connection_args
        self.__session = None

    @property
    def session(self) -> ClientSession:
        return self.__session

    async def __aenter__(self) -> "MCPTempClient":
        try:
            session = await MCPSessionManager().connect_to_server(
                server_name="tmp_client",
                exit_stack=self.exit_stack,
                **self.connection_args
            )
            self.__session = session
            return self
        except Exception:
            await self.exit_stack.aclose()
            raise

    async def __aexit__(
            self,
            exc_type: type[BaseException] | None,
            exc_val: BaseException | None,
            exc_tb: TracebackType | None,
    ) -> None:
        await self.exit_stack.aclose()


@singleton
class MCPSessionManager:
    """A manager class to manage different mcp server session."""

    def __init__(self):
        """Init an empty context variable dict and a thread lock used when
        add new key to this dict."""
        self.__mcp_session_dict = ContextVar("__mcp_session_dict__")
        self.__exit_stack = ContextVar("__mcp_exit_stack__")

    def init_session(self):
        self.__exit_stack.set(pick_exit_stack())
        self.__mcp_session_dict.set({})

    @property
    def mcp_session_dict(self) -> dict:
        if not self.__mcp_session_dict.get(None):
            self.__mcp_session_dict.set({})
        return self.__mcp_session_dict.get({})


    @property
    def exit_stack(self) -> AsyncExitStack:
        if not self.__exit_stack.get(None):
            self.__exit_stack.set(pick_exit_stack())
        return self.__exit_stack.get()

    async def clear_session(self):
        await self.exit_stack.aclose()
        self.__exit_stack.set(None)
        self.__mcp_session_dict.set(None)


    def save_mcp_session(self):
        return {
            'mcp_session_dict': self.__mcp_session_dict.get(None),
            'exit_stack': self.__exit_stack.get(None)
        }

    def recover_mcp_session(self, mcp_session_dict, exit_stack):
        self.__mcp_session_dict.set(mcp_session_dict)
        self.__exit_stack.set(exit_stack)

    def get_mcp_server_session_sync(
        self,
        server_name: str,
        transport: Literal["stdio", "sse", "websocket", "streamable_http"] = "stdio",
        **kwargs,
    ) -> ClientSession:
        if self.mcp_session_dict.get(server_name):
            return self.mcp_session_dict.get(server_name)
        return self.connect_to_server_sync(
            server_name=server_name,
            transport=transport,
            **kwargs
        )

    async def get_mcp_server_session(
        self,
        server_name: str,
        transport: Literal["stdio", "sse", "websocket", "streamable_http"] = "stdio",
        **kwargs,
    ) -> ClientSession:
        if self.mcp_session_dict.get(server_name):
            return self.mcp_session_dict.get(server_name)
        return await self.connect_to_server(
            server_name=server_name,
            transport=transport,
            **kwargs
        )

    async def connect_to_server(
        self,
        server_name: str,
        transport: Literal["stdio", "sse", "websocket", "streamable_http"] = "stdio",
        exit_stack: AsyncExitStack = None,
        **kwargs,
    ) -> ClientSession:
        """Connect to an MCP server using either stdio or SSE.
        Args:
            server_name: Name to identify this server connection
            transport: Type of transport to use ("stdio" or "sse"), defaults to "stdio"
            exit_stack: Use a temp exit_stack to get an temp session
            **kwargs: Additional arguments to pass to the specific connection method
        """
        if transport == "sse":
            if "url" not in kwargs:
                raise ValueError("'url' parameter is required for MCP SSE connection")
            session = await self.connect_to_server_via_sse(
                server_name,
                url=kwargs["url"],
                headers=kwargs.get("headers"),
                timeout=kwargs.get("timeout", DEFAULT_HTTP_TIMEOUT),
                sse_read_timeout=kwargs.get("sse_read_timeout", DEFAULT_SSE_READ_TIMEOUT),
                session_kwargs=kwargs.get("session_kwargs"),
                exit_stack=exit_stack
            )
        elif transport == "stdio":
            if "command" not in kwargs:
                raise ValueError("'command' parameter is required for stdio connection")
            if "args" not in kwargs:
                raise ValueError("'args' parameter is required for stdio connection")
            session = await self.connect_to_server_via_stdio(
                server_name,
                command=kwargs["command"],
                args=kwargs["args"],
                env=kwargs.get("env"),
                encoding=kwargs.get("encoding", DEFAULT_ENCODING),
                encoding_error_handler=kwargs.get(
                    "encoding_error_handler", DEFAULT_ENCODING_ERROR_HANDLER
                ),
                session_kwargs=kwargs.get("session_kwargs"),
                exit_stack=exit_stack
            )
        elif transport == "streamable_http":
            if "url" not in kwargs:
                raise ValueError(
                    "'url' parameter is required for MCP streamble_http connection")
            session = await self.connect_to_server_via_streamable_http(
                server_name,
                url=kwargs["url"],
                headers=kwargs.get("headers"),
                timeout=kwargs.get("timeout", DEFAULT_STREAMABLE_HTTP_TIMEOUT),
                sse_read_timeout=kwargs.get("sse_read_timeout",
                                            DEFAULT_STREAMABLE_HTTP_SSE_READ_TIMEOUT),
                session_kwargs=kwargs.get("session_kwargs"),
                exit_stack=exit_stack
            )
        elif transport == "websocket":
            if "url" not in kwargs:
                raise ValueError(
                    "'url' parameter is required for MCP websocket connection")
            session = await self.connect_to_server_via_websocket(
                server_name,
                url=kwargs["url"],
                session_kwargs=kwargs.get("session_kwargs"),
            )
        else:
            raise ValueError(f"Unsupported transport: {transport}. Must be 'stdio' or 'sse'")
        return session


    def connect_to_server_sync(
        self,
        server_name: str,
        transport: Literal["stdio", "sse", "websocket", "streamable_http"] = "stdio",
        **kwargs,
    ) -> ClientSession:
        """Connect to an MCP server using either stdio or SSE.
        Args:
            server_name: Name to identify this server connection
            transport: Type of transport to use ("stdio" or "sse"), defaults to "stdio"
            exit_stack: Use a temp exit_stack to get an temp session
            **kwargs: Additional arguments to pass to the specific connection method
        """
        if transport == "sse":
            if "url" not in kwargs:
                raise ValueError("'url' parameter is required for MCP SSE connection")
            session = self.connect_to_server_via_sse_sync(
                server_name,
                url=kwargs["url"],
                headers=kwargs.get("headers"),
                timeout=kwargs.get("timeout", DEFAULT_HTTP_TIMEOUT),
                sse_read_timeout=kwargs.get("sse_read_timeout", DEFAULT_SSE_READ_TIMEOUT),
                session_kwargs=kwargs.get("session_kwargs"),
            )
        elif transport == "stdio":
            if "command" not in kwargs:
                raise ValueError("'command' parameter is required for stdio connection")
            if "args" not in kwargs:
                raise ValueError("'args' parameter is required for stdio connection")
            session = self.connect_to_server_via_stdio_sync(
                server_name,
                command=kwargs["command"],
                args=kwargs["args"],
                env=kwargs.get("env"),
                encoding=kwargs.get("encoding", DEFAULT_ENCODING),
                encoding_error_handler=kwargs.get(
                    "encoding_error_handler", DEFAULT_ENCODING_ERROR_HANDLER
                ),
                session_kwargs=kwargs.get("session_kwargs")
            )
        elif transport == "streamable_http":
            if "url" not in kwargs:
                raise ValueError(
                    "'url' parameter is required for MCP streamble_http connection")
            session = self.connect_to_server_via_streamable_http_sync(
                server_name,
                url=kwargs["url"],
                headers=kwargs.get("headers"),
                timeout=kwargs.get("timeout", DEFAULT_STREAMABLE_HTTP_TIMEOUT),
                sse_read_timeout=kwargs.get("sse_read_timeout",
                                            DEFAULT_STREAMABLE_HTTP_SSE_READ_TIMEOUT),
                session_kwargs=kwargs.get("session_kwargs")
            )
        elif transport == "websocket":
            if "url" not in kwargs:
                raise ValueError(
                    "'url' parameter is required for MCP websocket connection")
            session = self.connect_to_server_via_websocket_sync(
                server_name,
                url=kwargs["url"],
                session_kwargs=kwargs.get("session_kwargs"),
            )
        else:
            raise ValueError(f"Unsupported transport: {transport}. Must be 'stdio' or 'sse'")
        return session


    async def connect_to_server_via_stdio(
        self,
        server_name: str,
        *,
        command: str,
        args: list[str],
        env: dict[str, str] | None = None,
        encoding: str = DEFAULT_ENCODING,
        encoding_error_handler: Literal[
            "strict", "ignore", "replace"
        ] = DEFAULT_ENCODING_ERROR_HANDLER,
        session_kwargs: dict | None = None,
        exit_stack: AsyncExitStack
    ) -> ClientSession:
        """Connect to a specific MCP server using stdio

        Args:
            server_name: Name to identify this server connection
            command: Command to execute
            args: Arguments for the command
            env: Environment variables for the command
            encoding: Character encoding
            encoding_error_handler: How to handle encoding errors
            session_kwargs: Additional keyword arguments to pass to the ClientSession
            exit_stack: Use a temp exit_stack to get an temp session
        """
        # NOTE: execution commands (e.g., `uvx` / `npx`) require PATH envvar to be set.
        # To address this, we automatically inject existing PATH envvar into the `env` value,
        # if it's not already set.
        env = env or {}
        if "PATH" not in env:
            env["PATH"] = os.environ.get("PATH", "")

        server_params = StdioServerParameters(
            command=command,
            args=args,
            encoding=encoding,
            encoding_error_handler=encoding_error_handler,
        )

        # Create and store the connection
        activate_exit_stack = exit_stack if exit_stack else self.exit_stack
        stdio_transport = await activate_exit_stack.enter_async_context(stdio_client(server_params))
        read, write = stdio_transport
        session_kwargs = session_kwargs or {}
        session = cast(
            ClientSession,
            await activate_exit_stack.enter_async_context(ClientSession(read, write, **session_kwargs)),
        )

        await session.initialize()
        if not exit_stack:
            self.mcp_session_dict[server_name] = session
        return session

    def connect_to_server_via_stdio_sync(
        self,
        server_name: str,
        *,
        command: str,
        args: list[str],
        env: dict[str, str] | None = None,
        encoding: str = DEFAULT_ENCODING,
        encoding_error_handler: Literal[
            "strict", "ignore", "replace"
        ] = DEFAULT_ENCODING_ERROR_HANDLER,
        session_kwargs: dict | None = None
    ) -> ClientSession:
        """Connect to a specific MCP server using stdio

        Args:
            server_name: Name to identify this server connection
            command: Command to execute
            args: Arguments for the command
            env: Environment variables for the command
            encoding: Character encoding
            encoding_error_handler: How to handle encoding errors
            session_kwargs: Additional keyword arguments to pass to the ClientSession
            exit_stack: Use a temp exit_stack to get an temp session
        """
        # NOTE: execution commands (e.g., `uvx` / `npx`) require PATH envvar to be set.
        # To address this, we automatically inject existing PATH envvar into the `env` value,
        # if it's not already set.
        env = env or {}
        if "PATH" not in env:
            env["PATH"] = os.environ.get("PATH", "")

        server_params = StdioServerParameters(
            command=command,
            args=args,
            encoding=encoding,
            encoding_error_handler=encoding_error_handler,
        )
        stdio_transport = self.exit_stack.enter_async_context(stdio_client(server_params))
        read, write = stdio_transport
        session_kwargs = session_kwargs or {}
        session = cast(
            ClientSession,
            self.exit_stack.enter_async_context(ClientSession(read, write, **session_kwargs)),
        )

        self.run_async(session.initialize)
        self.mcp_session_dict[server_name] = session
        return session

    async def connect_to_server_via_sse(
        self,
        server_name: str,
        *,
        url: str,
        headers: dict | None = None,
        timeout: float = DEFAULT_HTTP_TIMEOUT,
        sse_read_timeout: float = DEFAULT_SSE_READ_TIMEOUT,
        session_kwargs: dict | None = None,
        exit_stack: AsyncExitStack = None
    ) -> ClientSession:
        """Connect to a specific MCP server using SSE

        Args:
            server_name: Name to identify this server connection
            url: URL of the SSE server
            headers: HTTP headers to send to the SSE endpoint
            timeout: HTTP timeout
            sse_read_timeout: SSE read timeout
            session_kwargs: Additional keyword arguments to pass to the ClientSession
            exit_stack: Use a temp exit_stack to get an temp session
        """
        # Create and store the connection
        activate_exit_stack = exit_stack if exit_stack else self.exit_stack
        sse_transport = await activate_exit_stack.enter_async_context(
            sse_client(url, headers, timeout, sse_read_timeout)
        )
        read, write = sse_transport
        session_kwargs = session_kwargs or {}
        session = cast(
            ClientSession,
            await activate_exit_stack.enter_async_context(ClientSession(read, write, **session_kwargs)),
        )

        await session.initialize()
        if not exit_stack:
            self.mcp_session_dict[server_name] = session
        return session

    def connect_to_server_via_sse_sync(
        self,
        server_name: str,
        *,
        url: str,
        headers: dict | None = None,
        timeout: float = DEFAULT_HTTP_TIMEOUT,
        sse_read_timeout: float = DEFAULT_SSE_READ_TIMEOUT,
        session_kwargs: dict | None = None,
    ) -> ClientSession:
        """Connect to a specific MCP server using SSE

        Args:
            server_name: Name to identify this server connection
            url: URL of the SSE server
            headers: HTTP headers to send to the SSE endpoint
            timeout: HTTP timeout
            sse_read_timeout: SSE read timeout
            session_kwargs: Additional keyword arguments to pass to the ClientSession
        """
        # Create and store the connection
        sse_transport = self.exit_stack.enter_async_context(
            sse_client(url, headers, timeout, sse_read_timeout)
        )
        read, write = sse_transport
        session_kwargs = session_kwargs or {}
        session = cast(
            ClientSession,
            self.exit_stack.enter_async_context(ClientSession(read, write, **session_kwargs)),
        )

        self.run_async(session.initialize)
        self.mcp_session_dict[server_name] = session
        return session

    async def connect_to_server_via_streamable_http(
        self,
        server_name: str,
        *,
        url: str,
        headers: dict[str, Any] | None = None,
        timeout: timedelta = DEFAULT_STREAMABLE_HTTP_TIMEOUT,
        sse_read_timeout: timedelta = DEFAULT_STREAMABLE_HTTP_SSE_READ_TIMEOUT,
        session_kwargs: dict[str, Any] | None = None,
        exit_stack: AsyncExitStack = None
    ) -> ClientSession:
        """Connect to a specific MCP server using Streamable HTTP

        Args:
            server_name: Name to identify this server connection
            url: URL of the endpoint to connect to
            headers: HTTP headers to send to the endpoint
            timeout: HTTP timeout
            sse_read_timeout: How long (in seconds) the client will wait for a new event before disconnecting.
            session_kwargs: Additional keyword arguments to pass to the ClientSession
            exit_stack: Use a temp exit_stack to get an temp session
        """
        # Create and store the connection
        activate_exit_stack = exit_stack if exit_stack else self.exit_stack
        streamable_http_transport = await activate_exit_stack.enter_async_context(
            streamablehttp_client(url, headers, timeout, sse_read_timeout)
        )
        read, write, _ = streamable_http_transport
        session_kwargs = session_kwargs or {}
        session = cast(
            ClientSession,
            await activate_exit_stack.enter_async_context(ClientSession(read, write, **session_kwargs)),
        )

        await session.initialize()
        if not exit_stack:
            self.mcp_session_dict[server_name] = session
        return session

    def connect_to_server_via_streamable_http_sync(
            self,
            server_name: str,
            *,
            url: str,
            headers: dict[str, Any] | None = None,
            timeout: timedelta = DEFAULT_STREAMABLE_HTTP_TIMEOUT,
            sse_read_timeout: timedelta = DEFAULT_STREAMABLE_HTTP_SSE_READ_TIMEOUT,
            session_kwargs: dict[str, Any] | None = None
    ) -> ClientSession:
        """Connect to a specific MCP server using Streamable HTTP

        Args:
            server_name: Name to identify this server connection
            url: URL of the endpoint to connect to
            headers: HTTP headers to send to the endpoint
            timeout: HTTP timeout
            sse_read_timeout: How long (in seconds) the client will wait for a new event before disconnecting.
            session_kwargs: Additional keyword arguments to pass to the ClientSession
            exit_stack: Use a temp exit_stack to get an temp session
        """
        # Create and store the connection
        streamable_http_transport = self.exit_stack.enter_async_context(
            streamablehttp_client(url, headers, timeout, sse_read_timeout)
        )
        read, write, _ = streamable_http_transport
        session_kwargs = session_kwargs or {}
        session = cast(
            ClientSession,
            self.exit_stack.enter_async_context(
                ClientSession(read, write, **session_kwargs)),
        )
        self.run_async(session.initialize)
        self.mcp_session_dict[server_name] = session
        return session

    async def connect_to_server_via_websocket(
        self,
        server_name: str,
        *,
        url: str,
        session_kwargs: dict[str, Any] | None = None,
        exit_stack: AsyncExitStack = None
    ) -> ClientSession:
        """Connect to a specific MCP server using Websockets

        Args:
            server_name: Name to identify this server connection
            url: URL of the Websocket endpoint
            session_kwargs: Additional keyword arguments to pass to the ClientSession
            exit_stack: Use a temp exit_stack to get an temp session
        Raises:
            ImportError: If websockets package is not installed
        """
        activate_exit_stack = exit_stack if exit_stack else self.exit_stack
        try:
            from mcp.client.websocket import websocket_client
        except ImportError:
            raise ImportError(
                "Could not import websocket_client. ",
                "To use Websocket connections, please install the required dependency with: ",
                "'pip install mcp[ws]' or 'pip install websockets'",
            ) from None

        ws_transport = await activate_exit_stack.enter_async_context(websocket_client(url))
        read, write = ws_transport
        session_kwargs = session_kwargs or {}
        session = cast(
            ClientSession,
            await activate_exit_stack.enter_async_context(ClientSession(read, write, **session_kwargs)),
        )

        await session.initialize()
        if not exit_stack:
            self.mcp_session_dict[server_name] = session
        return session

    def connect_to_server_via_websocket_sync(
        self,
        server_name: str,
        *,
        url: str,
        session_kwargs: dict[str, Any] | None = None
    ) -> ClientSession:
        """Connect to a specific MCP server using Websockets

        Args:
            server_name: Name to identify this server connection
            url: URL of the Websocket endpoint
            session_kwargs: Additional keyword arguments to pass to the ClientSession
        Raises:
            ImportError: If websockets package is not installed
        """
        try:
            from mcp.client.websocket import websocket_client
        except ImportError:
            raise ImportError(
                "Could not import websocket_client. ",
                "To use Websocket connections, please install the required dependency with: ",
                "'pip install mcp[ws]' or 'pip install websockets'",
            ) from None

        ws_transport = self.exit_stack.enter_async_context(websocket_client(url))
        read, write = ws_transport
        session_kwargs = session_kwargs or {}
        session = cast(
            ClientSession,
            self.exit_stack.enter_async_context(ClientSession(read, write, **session_kwargs)),
        )

        self.run_async(session.initialize)
        self.mcp_session_dict[server_name] = session
        return session

    def safe_close_stack(self) -> None:
        try:
            loop = asyncio.get_running_loop()
        except RuntimeError:
            if isinstance(self.exit_stack, SyncAsyncExitStack):
                self.exit_stack.close()
            elif isinstance(self.exit_stack, AsyncExitStack):
                asyncio.run(self.exit_stack.aclose())
        else:
            if isinstance(self.exit_stack, AsyncExitStack):
                loop.create_task(self.exit_stack.aclose())
            elif isinstance(self.exit_stack, SyncAsyncExitStack):
                self.exit_stack.close()
        self.__exit_stack.set(None)
        self.__mcp_session_dict.set(None)

    def run_async(self, func, *args, **kwargs):
        return self.exit_stack.run_async(func, *args, **kwargs)



#!/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2025/03/31
# @Author  : zhangdongxu
# @Email   : zhangdongxu0852@163.com
# @FileName: readimage_tool.py

import cv2
import pytesseract
from PIL import Image
import numpy as np
import re
import os

def enhance_image(image):
    """
    Enhance the image: convert to grayscale, apply CLAHE for contrast enhancement,
    then apply bilateral filtering to reduce noise while preserving edges.
    """
    gray = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)
    clahe = cv2.createCLAHE(clipLimit=2.0, tileGridSize=(8, 8))
    enhanced = clahe.apply(gray)
    # Bilateral filtering reduces noise while preserving edge details
    filtered = cv2.bilateralFilter(enhanced, 9, 75, 75)
    return filtered

def detect_text_regions(image, east_model='frozen_east_text_detection.pb', min_confidence=0.5, width=320, height=320):
    """
    Detect text regions in the image using the EAST text detection model.
    Returns a list of image regions (as numpy arrays) corresponding to text areas.
    """
    # Load the EAST model
    net = cv2.dnn.readNet(east_model)
    orig = image.copy()
    (origH, origW) = image.shape[:2]

    # The EAST model requires dimensions to be multiples of 32.
    newW, newH = width, height
    rW = origW / float(newW)
    rH = origH / float(newH)
    resized = cv2.resize(image, (newW, newH))

    blob = cv2.dnn.blobFromImage(resized, 1.0, (newW, newH),
                                 (123.68, 116.78, 103.94), swapRB=True, crop=False)
    net.setInput(blob)
    # Output layers of the EAST model
    layerNames = ["feature_fusion/Conv_7/Sigmoid", "feature_fusion/concat_3"]
    (scores, geometry) = net.forward(layerNames)
    (numRows, numCols) = scores.shape[2:4]
    rects = []
    confidences = []

    # Iterate over rows and columns to extract predictions
    for y in range(0, numRows):
        scoresData = scores[0, 0, y]
        xData0 = geometry[0, 0, y]
        xData1 = geometry[0, 1, y]
        xData2 = geometry[0, 2, y]
        xData3 = geometry[0, 3, y]
        anglesData = geometry[0, 4, y]

        for x in range(0, numCols):
            if scoresData[x] < min_confidence:
                continue

            offsetX = x * 4.0
            offsetY = y * 4.0

            angle = anglesData[x]
            cos = np.cos(angle)
            sin = np.sin(angle)

            h = xData0[x] + xData2[x]
            w = xData1[x] + xData3[x]

            endX = int(offsetX + (cos * xData1[x]) + (sin * xData2[x]))
            endY = int(offsetY - (sin * xData1[x]) + (cos * xData2[x]))
            startX = int(endX - w)
            startY = int(endY - h)

            rects.append((startX, startY, endX, endY))
            confidences.append(float(scoresData[x]))

    # Apply non-maxima suppression to suppress overlapping boxes
    boxes = cv2.dnn.NMSBoxes(rects, confidences, min_confidence, 0.4)
    regions = []
    if len(boxes) > 0:
        for i in boxes.flatten():
            (startX, startY, endX, endY) = rects[i]
            # Scale the bounding box coordinates back to the original image dimensions
            startX = int(startX * rW)
            startY = int(startY * rH)
            endX = int(endX * rW)
            endY = int(endY * rH)
            # Crop the detected text region from the original image
            region = orig[startY:endY, startX:endX]
            regions.append(region)
    return regions

def ocr_on_regions(regions, lang='chi_sim+eng'):
    """
    Perform OCR on each text region separately and concatenate the results.
    """
    texts = []
    config = "--oem 3 --psm 6"
    for region in regions:
        if region.size == 0:
            continue
        # Optionally, additional preprocessing (e.g., binarization) can be applied here.
        pil_img = Image.fromarray(region)
        text = pytesseract.image_to_string(pil_img, lang=lang, config=config)
        texts.append(text)
    return "\n".join(texts)

def clean_extracted_text(text):
    """
    Clean the OCR output text by removing extra whitespace.
    """
    text = re.sub(r'\s+', ' ', text)
    return text.strip()

def save_text_to_file(text, output_file='extracted_text.txt'):
    with open(output_file, 'w', encoding='utf-8') as f:
        f.write(text)
    print(f"Text content saved to {output_file}")

def extract_text_from_image(image_path, use_east=True, lang='chi_sim+eng'):
    """
    Extract text from an image:
      - First, enhance the image.
      - If use_east is True, use the EAST model to detect text regions and perform OCR on each region.
      - Otherwise, perform OCR on the enhanced whole image.
    """
    # Read image using IMREAD_UNCHANGED to support any image format
    image = cv2.imread(image_path, cv2.IMREAD_UNCHANGED)
    if image is None:
        raise ValueError("Cannot load image, please check the path")
    
    # Convert image to 3-channel BGR if it's not already
    if len(image.shape) == 2:
        # If the image is grayscale, convert to BGR
        image = cv2.cvtColor(image, cv2.COLOR_GRAY2BGR)
    elif len(image.shape) == 3 and image.shape[2] == 4:
        # If the image has an alpha channel, convert BGRA to BGR
        image = cv2.cvtColor(image, cv2.COLOR_BGRA2BGR)
    
    # Enhance the image
    enhanced = enhance_image(image)
    
    if use_east:
        try:
            # Use the original color image for EAST detection (usually yields better results)
            regions = detect_text_regions(cv2.cvtColor(image, cv2.COLOR_BGR2RGB))
            if len(regions) == 0:
                raise Exception("No text regions detected, falling back to full image OCR")
            raw_text = ocr_on_regions(regions, lang=lang)
        except Exception as e:
            print("EAST detection failed, using full image OCR:", e)
            pil_img = Image.fromarray(enhanced)
            config = "--oem 3 --psm 6"
            raw_text = pytesseract.image_to_string(pil_img, lang=lang, config=config)
    else:
        pil_img = Image.fromarray(enhanced)
        config = "--oem 3 --psm 6"
        raw_text = pytesseract.image_to_string(pil_img, lang=lang, config=config)
    
    final_text = clean_extracted_text(raw_text)
    return final_text

if __name__ == "__main__":
    # Specify the input image file. It can be a jpg, png, or any common format.
    image_file = 'input.png'  # Change to the appropriate image file as needed
    text_output = extract_text_from_image(image_file, use_east=True, lang='chi_sim+eng')
    print("Extracted text:")
    print(text_output)
    save_text_to_file(text_output, 'extracted_text.txt')



# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2025/3/22 18:00
# @Author  : hiro
# @Email   : hiromesh@qq.com
# @FileName: command_status_tool.py

import json
from agentuniverse.agent.action.tool.tool import Tool, ToolInput
from agentuniverse.agent.action.tool.common_tool.run_command_tool import get_command_result


class CommandStatusTool(Tool):
    def execute(self, thread_id: int) -> str:
        if isinstance(thread_id, str) and thread_id.isdigit():
            thread_id = int(thread_id)

        result = get_command_result(thread_id)

        if result is None:
            return json.dumps({
                "error": f"No command found with thread_id: {thread_id}",
                "status": "not_found"
            })
        return result.message



# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/12/17 11:00
# @Author  : fanen.lhy
# @Email   : fanen.lhy@antgroup.com
# @FileName: context_archive_utils.py
import re
from agentuniverse.base.context.framework_context_manager import FrameworkContextManager


def get_current_context_archive():
    context_archive = FrameworkContextManager().get_context(
        'context_archive', None)
    if not context_archive:
        context_archive = {}
        FrameworkContextManager().set_context('context_archive', {})

    return context_archive


def update_context_archive(name, data, description):
    react_memory = get_current_context_archive()
    react_memory[name] = {
        'data': data,
        'description': description
    }


# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2025/4/8 11:36
# @Author  : wangchongshi
# @Email   : wangchongshi.wcs@antgroup.com
# @FileName: zhipu_official_llm_channel.py
from typing import Optional

from agentuniverse.llm.llm_channel.llm_channel import LLMChannel

ZHIPU_MAX_CONTEXT_LENGTH = {
    "GLM-4-Plus": 128000,
    "GLM-4-0520": 128000,
    "GLM-4-AirX": 8000,
    "GLM-4-Air": 128000,
    "GLM-4-Long": 1000000,
    "GLM-4-Flash": 128000,
    "GLM-4": 128000,
}


class ZhiPuOfficialLLMChannel(LLMChannel):
    channel_api_base: Optional[str] = "https://open.bigmodel.cn/api/paas/v4/"

    def max_context_length(self) -> int:
        """Max context length.

          The total length of input tokens and generated tokens is limited by the openai model's context length.
          """
        return ZHIPU_MAX_CONTEXT_LENGTH.get(self.channel_model_name, 128000)
